#ifndef __lifespan_h__
#define __lifespan_h__

#include <OS.h>

class Lifespan
{
	sem_id			mSemID;

public:

	Lifespan() { mSemID = B_ERROR; }
	~Lifespan()
	{
		if (B_ERROR < mSemID)
			::release_sem(mSemID);
	}

	typedef		sem_id		lifesign_t;

	// method creates and returns a lifesign for 'this' object.
	lifesign_t CreateLifesign()
	{
		if (mSemID < 0)
			mSemID = ::create_sem(0, NULL);
	
		return (lifesign_t) mSemID;
	}

	// method blocks until object w/ specified lifesign gets destroyed.
	static status_t WaitForDeath(lifesign_t sign)
	{
		status_t acq_res = ::acquire_sem((sem_id) sign);
	
		if (B_OK == acq_res)
			::delete_sem((sem_id) sign);
	
		return acq_res;			
	}

// note: once CreateLifesign() was successfully invoked for an object, it should
// be succeeded by a respective WaitForDeath(), otherwise the thread would leak
// a semaphore!
};

#endif // __lifespan_h__
